@@ -1,6 +1,6 @@
 /******************************************************************************
 *   Copyright 2008 Aaron Seigo <aseigo@kde.org>                               *
-*   Copyright 2012-2013 Sebastian Kügler <sebas@kde.org>                      *
+*   Copyright 2012-2017 Sebastian Kügler <sebas@kde.org>                      *
 *                                                                             *
 *   This library is free software; you can redistribute it and/or             *
 *   modify it under the terms of the GNU Library General Public               *
@@ -35,17 +35,21 @@
 
 #include <qcommandlineparser.h>
 #include <QDir>
+#include <QDirIterator>
 #include <QFileInfo>
 #include <QMap>
 #include <QRegularExpression>
 #include <QStandardPaths>
 #include <QStringList>
 #include <QTimer>
+#include <QUrl>
 #include <QXmlStreamWriter>
 
 #include <iostream>
 #include <iomanip>
 
+#include "options.h"
+
 #include "../kpackage/config-package.h"
 //for the index creation function
 #include "../kpackage/private/packagejobthread_p.h"
@@ -69,7 +73,7 @@ static QVector<KPluginMetaData> listPackageTypes()
 
     QVector<KPluginMetaData> offers;
     Q_FOREACH (const QString &plugindir, libraryPaths) {
-        const QString &_ixfile = plugindir + QStringLiteral("kpluginindex.json");
+        const QString &_ixfile = plugindir + s_kpluginindex;
         QFile indexFile(_ixfile);
         if (indexFile.exists()) {
             indexFile.open(QIODevice::ReadOnly);
@@ -111,6 +115,7 @@ public:
     void renderTypeTable(const QMap<QString, QStringList> &plugins);
     void listTypes();
     void coutput(const QString &msg);
+    void cerror(const QString &msg);
     QCommandLineParser *parser;
 };
 
@@ -129,10 +134,10 @@ PackageTool::~PackageTool()
 
 void PackageTool::runMain()
 {
-    KPackage::PackageStructure *structure = new KPackage::PackageStructure;
-    if (d->parser->isSet(QStringLiteral("hash"))) {
-        const QString path = d->parser->value(QStringLiteral("hash"));
-        KPackage::Package package(structure);
+    KPackage::PackageStructure structure;
+    if (d->parser->isSet(Options::hash)) {
+        const QString path = d->parser->value(Options::hash);
+        KPackage::Package package(&structure);
         package.setPath(path);
         const QString hash =
             QString::fromLocal8Bit(package.cryptographicHash(QCryptographicHash::Sha1));
@@ -146,43 +151,43 @@ void PackageTool::runMain()
         return;
     }
 
-    if (d->parser->isSet(QStringLiteral("list-types"))) {
+    if (d->parser->isSet(Options::listTypes)) {
         d->listTypes();
         exit(0);
         return;
     }
 
-    QString type = d->parser->value(QStringLiteral("type"));
+    QString type = d->parser->value(Options::type);
     d->pluginTypes.clear();
     d->installer = Package();
 
-    if (d->parser->isSet(QStringLiteral("remove"))) {
-        d->package = d->parser->value(QStringLiteral("remove"));
-    } else if (d->parser->isSet(QStringLiteral("upgrade"))) {
-        d->package = d->parser->value(QStringLiteral("upgrade"));
-    } else if (d->parser->isSet(QStringLiteral("install"))) {
-        d->package = d->parser->value(QStringLiteral("install"));
-    } else if (d->parser->isSet(QStringLiteral("show"))) {
-        d->package = d->parser->value(QStringLiteral("show"));
-    } else if (d->parser->isSet(QStringLiteral("appstream-metainfo"))) {
-        d->package = d->parser->value(QStringLiteral("appstream-metainfo"));
+    if (d->parser->isSet(Options::remove)) {
+        d->package = d->parser->value(Options::remove);
+    } else if (d->parser->isSet(Options::upgrade)) {
+        d->package = d->parser->value(Options::upgrade);
+    } else if (d->parser->isSet(Options::install)) {
+        d->package = d->parser->value(Options::install);
+    } else if (d->parser->isSet(Options::show)) {
+        d->package = d->parser->value(Options::show);
+    } else if (d->parser->isSet(Options::appstream)) {
+        d->package = d->parser->value(Options::appstream);
     }
 
     if (!QDir::isAbsolutePath(d->package)) {
-        d->packageFile = QDir(QDir::currentPath() + '/' + d->package).absolutePath();
+        d->packageFile = QDir(QDir::currentPath() + QLatin1Char('/') + d->package).absolutePath();
         d->packageFile = QFileInfo(d->packageFile).canonicalFilePath();
-        if (d->parser->isSet(QStringLiteral("upgrade"))) {
+        if (d->parser->isSet(Options::upgrade)) {
             d->package = d->packageFile;
         }
     } else {
         d->packageFile = d->package;
     }
 
-    if (!d->packageFile.isEmpty() && (!d->parser->isSet(QStringLiteral("type")) ||
+    if (!d->packageFile.isEmpty() && (!d->parser->isSet(Options::type) ||
                                       type.compare(i18nc("package type", "wallpaper"), Qt::CaseInsensitive) == 0 ||
                                       type.compare(QLatin1String("wallpaper"), Qt::CaseInsensitive) == 0)) {
         // Check type for common plasma packages
-        KPackage::Package package(structure);
+        KPackage::Package package(&structure);
         QString serviceType;
         package.setPath(d->packageFile);
 
@@ -217,26 +222,30 @@ void PackageTool::runMain()
         d->packageRoot = d->installer.defaultPackageRoot();
         d->pluginTypes << type;
     }
-    if (d->parser->isSet(QStringLiteral("show"))) {
+    if (d->parser->isSet(Options::show)) {
         const QString pluginName = d->package;
         showPackageInfo(pluginName);
         return;
-    } else if (d->parser->isSet(QStringLiteral("appstream-metainfo"))) {
+    } else if (d->parser->isSet(Options::appstream)) {
         const QString pluginName = d->package;
         showAppstreamInfo(pluginName);
         return;
     }
 
-    if (d->parser->isSet(QStringLiteral("list"))) {
+    if (d->parser->isSet(Options::list)) {
         d->packageRoot = findPackageRoot(d->package, d->packageRoot);
-        d->coutput(i18n("Listing service types: %1 in %2", d->pluginTypes.join(QByteArray(", ")), d->packageRoot));
+        d->coutput(i18n("Listing service types: %1 in %2", d->pluginTypes.join(QStringLiteral(", ")), d->packageRoot));
         listPackages(d->pluginTypes, d->packageRoot);
         exit(0);
 
-    } else if (d->parser->isSet(QStringLiteral("generate-index"))) {
+    } else if (d->parser->isSet(Options::generateIndex)) {
         recreateIndex();
         exit(0);
 
+    } else if (d->parser->isSet(Options::removeIndex)) {
+        removeIndex();
+        exit(0);
+
     } else {
         // install, remove or upgrade
         if (!d->installer.isValid()) {
@@ -246,7 +255,7 @@ void PackageTool::runMain()
 
         d->packageRoot = findPackageRoot(d->package, d->packageRoot);
 
-        if (d->parser->isSet(QStringLiteral("remove")) || d->parser->isSet(QStringLiteral("upgrade"))) {
+        if (d->parser->isSet(Options::remove) || d->parser->isSet(Options::upgrade)) {
             QString pkgPath;
             foreach (const QString &t, d->pluginTypes) {
                 KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(t);
@@ -263,18 +272,18 @@ void PackageTool::runMain()
                 pkgPath = d->package;
             }
 
-            if (d->parser->isSet(QStringLiteral("upgrade"))) {
+            if (d->parser->isSet(Options::upgrade)) {
                 d->installer.setPath(d->package);
             }
             QString _p = d->packageRoot;
-            if (!_p.endsWith('/')) {
-                _p.append('/');
+            if (!_p.endsWith(QLatin1Char('/'))) {
+                _p.append(QLatin1Char('/'));
             }
             _p.append(d->package);
             d->installer.setDefaultPackageRoot(d->packageRoot);
             d->installer.setPath(pkgPath);
 
-            if (!d->parser->isSet(QStringLiteral("type"))) {
+            if (!d->parser->isSet(Options::type)) {
                 foreach (const QString &st, d->installer.metadata().serviceTypes()) {
                     if (!d->pluginTypes.contains(st)) {
                         d->pluginTypes << st;
@@ -316,7 +325,7 @@ void PackageTool::runMain()
                 exit(2);
             }
         }
-        if (d->parser->isSet(QStringLiteral("install"))) {
+        if (d->parser->isSet(Options::install)) {
             KJob *installJob = d->installer.install(d->packageFile, d->packageRoot);
             connect(installJob, SIGNAL(result(KJob*)), SLOT(packageInstalled(KJob*)));
             return;
@@ -333,6 +342,11 @@ void PackageToolPrivate::coutput(const QString &msg)
     *cout << msg << endl;
 }
 
+void PackageToolPrivate::cerror(const QString &msg)
+{
+    *cerr << msg << endl;
+}
+
 QStringList PackageToolPrivate::packages(const QStringList &types, const QString &path)
 {
     QStringList result;
@@ -353,7 +367,7 @@ QStringList PackageToolPrivate::packages(const QStringList &types, const QString
 void PackageTool::showPackageInfo(const QString &pluginName)
 {
     QString type = QStringLiteral("KPackage/Generic");
-    if (!d->pluginTypes.contains(type) && d->pluginTypes.count() > 0) {
+    if (!d->pluginTypes.contains(type) && !d->pluginTypes.isEmpty()) {
         type = d->pluginTypes.at(0);
     }
     KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(type);
@@ -406,7 +420,7 @@ bool translateKPluginToAppstream(const QString &tagName, const QString &configFi
             }
 
             writer.writeStartElement(tagName);
-            writer.writeAttribute("xml:lang", match.captured(1));
+            writer.writeAttribute(QStringLiteral("xml:lang"), match.captured(1));
             writer.writeCharacters(content);
             writer.writeEndElement();
         }
@@ -420,13 +434,14 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
     KPluginMetaData packageStructureMetaData;
     //if the path passed is an absolute path, and a metadata file is found under it, use that metadata file to generate the appstream info.
     // This can happen in the case an application wanting to support kpackage based extensions includes in the same project both the packagestructure plugin and the packages themselves. In that case at build time the packagestructure plugin wouldn't be installed yet
+
     if (QFile::exists(pluginName + QStringLiteral("/metadata.desktop"))) {
         i = KPluginMetaData(pluginName + QStringLiteral("/metadata.desktop"));
     } else if (QFile::exists(pluginName + QStringLiteral("/metadata.json"))) {
         i = KPluginMetaData(pluginName + QStringLiteral("/metadata.json"));
     } else {
         QString type = QStringLiteral("KPackage/Generic");
-        if (!d->pluginTypes.contains(type) && d->pluginTypes.count() > 0) {
+        if (!d->pluginTypes.contains(type) && !d->pluginTypes.isEmpty()) {
             type = d->pluginTypes.at(0);
         }
         KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(type);
@@ -457,57 +472,93 @@ void PackageTool::showAppstreamInfo(const QString &pluginName)
         return;
     }
 
-    if (i.rawData().value("NoDisplay").toBool(false)) {
-        *cerr << i18n("%1 has NoDisplay enabled\n", pluginName);
-        std::exit(0);
-    }
-
-    QString parentApp = i.rawData().value("X-KDE-ParentApp").toString();
+    QString parentApp = i.rawData().value(QLatin1String("X-KDE-ParentApp")).toString();
     if (parentApp.isEmpty()) {
-        parentApp = packageStructureMetaData.rawData().value("X-KDE-ParentApp").toString();
+        parentApp = packageStructureMetaData.rawData().value(QLatin1String("X-KDE-ParentApp")).toString();
     }
     const QJsonObject rootObject = i.rawData()[QStringLiteral("KPlugin")].toObject();
 
+    // Be super aggressive in finding a NoDisplay property. It can be a top-level property or
+    // inside the KPlugin object, it also can be either a stringy type or a bool type. Try all
+    // possible scopes and type conversions to find NoDisplay
+    for (const auto& object : { i.rawData(), rootObject }) {
+        if (object.value(QLatin1String("NoDisplay")).toBool(false) ||
+                // Standard value is unprocessed string we'll need to deal with.
+                object.value(QLatin1String("NoDisplay")).toString() == QStringLiteral("true")) {
+            std::exit(0);
+        }
+    }
+
     QXmlStreamAttributes componentAttributes;
     if (!parentApp.isEmpty()) {
-        componentAttributes << QXmlStreamAttribute("type", "addon");
+        componentAttributes << QXmlStreamAttribute(QLatin1String("type"), QLatin1String("addon"));
     }
 
-    QXmlStreamWriter writer(cout->device());
+    // Compatibility: without appstream-metainfo-output argument we print the XML output to STDOUT
+    // with the argument we'll print to the defined path.
+    // TODO: in KF6 we should switch to argument-only.
+    QIODevice *outputDevice = cout->device();
+    QScopedPointer<QFile> outputFile;
+    const auto outputPath = d->parser->value(Options::appstreamOutput);
+    if (!outputPath.isEmpty()) {
+        auto outputUrl = QUrl::fromUserInput(outputPath);
+        outputFile.reset(new QFile(outputUrl.toLocalFile()));
+        if (!outputFile->open(QFile::WriteOnly | QFile::Text)) {
+            *cerr << "Failed to open output file for writing.";
+            exit(1);
+        }
+        outputDevice = outputFile.data();
+    }
+
+    if (!rootObject.contains(QStringLiteral("Description"))) {
+        *cerr << "Error: description missing, will result in broken appdata field as <sumary/> is mandatory at " << QFileInfo(i.metaDataFileName()).absoluteFilePath();
+        std::exit(10);
+    }
+
+    QXmlStreamWriter writer(outputDevice);
     writer.setAutoFormatting(true);
     writer.writeStartDocument();
-    writer.writeStartElement("component");
+    writer.writeStartElement(QStringLiteral("component"));
     writer.writeAttributes(componentAttributes);
 
-    writer.writeTextElement("id", i.pluginId());
+    writer.writeTextElement(QStringLiteral("id"), i.pluginId());
     if (!parentApp.isEmpty()) {
-        writer.writeTextElement("extends", parentApp);
+        writer.writeTextElement(QStringLiteral("extends"), parentApp);
     }
-    translateKPluginToAppstream("name", "Name", rootObject, writer, false);
-    translateKPluginToAppstream("summary", "Description", rootObject, writer, false);
+    translateKPluginToAppstream(QStringLiteral("name"), QStringLiteral("Name"), rootObject, writer, false);
+    translateKPluginToAppstream(QStringLiteral("summary"), QStringLiteral("Description"), rootObject, writer, false);
     if (!i.website().isEmpty()) {
-        writer.writeStartElement("url");
-        writer.writeAttribute("type", "homepage");
+        writer.writeStartElement(QStringLiteral("url"));
+        writer.writeAttribute(QStringLiteral("type"), QStringLiteral("homepage"));
         writer.writeCharacters(i.website());
         writer.writeEndElement();
     }
+
+    if (i.pluginId().startsWith(QLatin1String("org.kde."))) {
+        writer.writeStartElement(QStringLiteral("url"));
+        writer.writeAttribute(QStringLiteral("type"), QStringLiteral("donation"));
+        writer.writeCharacters(QStringLiteral("https://www.kde.org/donate.php?app=%1").arg(i.pluginId()));
+        writer.writeEndElement();
+    }
+
     const auto authors = i.authors();
     if (!authors.isEmpty()) {
         QStringList authorsText;
+        authorsText.reserve(authors.size());
         for (const auto &author: authors) {
             authorsText += QStringLiteral("%1 <%2>").arg(author.name(), author.emailAddress());
         }
-        writer.writeTextElement("developer_name", authorsText.join(", "));
+        writer.writeTextElement(QStringLiteral("developer_name"), authorsText.join(QStringLiteral(", ")));
     }
 
     if (!i.iconName().isEmpty()) {
         writer.writeStartElement(QStringLiteral("icon"));
-        writer.writeAttribute("type", "stock");
+        writer.writeAttribute(QStringLiteral("type"), QStringLiteral("stock"));
         writer.writeCharacters(i.iconName());
         writer.writeEndElement();
     }
-    writer.writeTextElement("project_license", i.license());
-    writer.writeTextElement("metadata_license", "CC0-1.0");
+    writer.writeTextElement(QStringLiteral("project_license"), KAboutLicense::byKeyword(i.license()).spdx());
+    writer.writeTextElement(QStringLiteral("metadata_license"), QStringLiteral("CC0-1.0"));
     writer.writeEndElement();
     writer.writeEndDocument();
 
@@ -519,13 +570,13 @@ QString PackageTool::findPackageRoot(const QString &pluginName, const QString &p
     Q_UNUSED(pluginName);
     Q_UNUSED(prefix);
     QString packageRoot;
-    if (d->parser->isSet(QStringLiteral("packageroot")) && d->parser->isSet(QStringLiteral("global")) && !d->parser->isSet(QStringLiteral("generate-index"))) {
+    if (d->parser->isSet(Options::packageRoot) && d->parser->isSet(Options::global) && !d->parser->isSet(Options::generateIndex)) {
         qWarning() << i18nc("The user entered conflicting options packageroot and global, this is the error message telling the user he can use only one", "The packageroot and global options conflict with each other, please select only one.");
         ::exit(7);
-    } else if (d->parser->isSet(QStringLiteral("packageroot"))) {
-        packageRoot = d->parser->value(QStringLiteral("packageroot"));
+    } else if (d->parser->isSet(Options::packageRoot)) {
+        packageRoot = d->parser->value(Options::packageRoot);
         //qDebug() << "(set via arg) d->packageRoot is: " << d->packageRoot;
-    } else if (d->parser->isSet(QStringLiteral("global"))) {
+    } else if (d->parser->isSet(Options::global)) {
         auto const paths = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, d->packageRoot, QStandardPaths::LocateDirectory);
         if (!paths.isEmpty()) {
             packageRoot = paths.last();
@@ -597,8 +648,8 @@ void PackageToolPrivate::listTypes()
     coutput(i18n("Built in:"));
 
     QMap<QString, QStringList> builtIns;
-    builtIns.insert(i18n("KPackage/Generic"), QStringList() << QStringLiteral("KPackage/Generic") << KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/packages/");
-    builtIns.insert(i18n("KPackage/GenericQML"), QStringList() << QStringLiteral("KPackage/GenericQML") << KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/packagesqml/");
+    builtIns.insert(i18n("KPackage/Generic"), QStringList() << QStringLiteral("KPackage/Generic") << QStringLiteral(KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/packages/"));
+    builtIns.insert(i18n("KPackage/GenericQML"), QStringList() << QStringLiteral("KPackage/GenericQML") << QStringLiteral(KPACKAGE_RELATIVE_DATA_INSTALL_DIR "/genericqml/"));
 
     renderTypeTable(builtIns);
 
@@ -610,14 +661,11 @@ void PackageToolPrivate::listTypes()
 
         QMap<QString, QStringList> plugins;
         foreach (const KPluginMetaData &info, offers) {
-            //const QString proot = "";
             KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(info.pluginId());
             QString name = info.name();
             QString plugin = info.pluginId();
             QString path = pkg.defaultPackageRoot();
-            //QString path = defaultPackageRoot;
             plugins.insert(name, QStringList() << plugin << path);
-            //qDebug() << "KService stuff:" << name << plugin << comment << path;
         }
 
         renderTypeTable(plugins);
@@ -629,10 +677,34 @@ void PackageTool::recreateIndex()
     d->packageRoot = findPackageRoot(d->package, d->packageRoot);
 
     if (!QDir::isAbsolutePath(d->packageRoot)) {
-        if (d->parser->isSet(QStringLiteral("global"))) {
+        if (d->parser->isSet(Options::global)) {
+
+            // Find package roots
+            QStringList packageRoots;
+            const QVector<KPluginMetaData> offers = listPackageTypes();
+            if (!offers.isEmpty()) {
+                foreach (const KPluginMetaData &info, offers) {
+                    KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(info.pluginId());
+                    const auto proot = pkg.defaultPackageRoot();
+                    if (!proot.isEmpty()) {
+                        packageRoots << pkg.defaultPackageRoot();
+                    }
+                }
+            }
+
             Q_FOREACH(auto const &p, QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, d->packageRoot, QStandardPaths::LocateDirectory)) {
-                d->coutput(i18n("Generating %1/kpluginindex.json", p));
-                KPackage::indexDirectory(p, QStringLiteral("kpluginindex.json"));
+                for (const auto &_proot: qAsConst(packageRoots)) {
+                    QFileInfo packageDirInfo(QStringLiteral("%1%2").arg(p, _proot));
+                    if (!packageDirInfo.isWritable()) {
+                        continue;
+                    }
+                    const QString packagedir = packageDirInfo.absoluteFilePath();
+                    if (KPackage::indexDirectory(packagedir, s_kpluginindex)) {
+                        d->coutput(i18n("Generating %1%2", packagedir, s_kpluginindex));
+                    } else {
+                        d->cerror(i18n("Didn't write %1%2", packagedir, s_kpluginindex));
+                    }
+                }
             }
             return;
         } else {
@@ -640,8 +712,46 @@ void PackageTool::recreateIndex()
         }
     }
 
-    d->coutput(i18n("Generating %1/kpluginindex.json", d->packageRoot));
-    KPackage::indexDirectory(d->packageRoot, QStringLiteral("kpluginindex.json"));
+    if (KPackage::indexDirectory(d->packageRoot, s_kpluginindex)) {
+        d->coutput(i18n("Generating %1/%2", d->packageRoot, s_kpluginindex));
+    } else {
+        d->cerror(i18n("Cannot write %1/%2", d->packageRoot, s_kpluginindex));
+    }
+}
+
+void PackageTool::removeIndex()
+{
+    d->packageRoot = findPackageRoot(d->package, d->packageRoot);
+
+    if (!QDir::isAbsolutePath(d->packageRoot)) {
+        if (d->parser->isSet(Options::global)) {
+            Q_FOREACH(auto const &p, QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, d->packageRoot, QStandardPaths::LocateDirectory)) {
+                QDirIterator it(p, QStringList(s_kpluginindex), QDir::Files | QDir::Writable | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
+                qDebug() << "IX index remove" << p;
+                while (it.hasNext()) {
+                    it.next();
+                    const QString path = it.fileInfo().absoluteFilePath();
+                    QFile file(path);
+                    if (!file.remove()) {
+                        d->cerror(i18n("Could not remove index file %1", file.fileName()));
+                    } else {
+                        d->coutput(i18n("Removed %1", file.fileName()));
+                    }
+                }
+            }
+            return;
+        } else {
+            d->packageRoot = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + d->packageRoot;
+        }
+    }
+    QFile file(d->packageRoot + s_kpluginindex);
+    if (file.exists()) {
+        if (!file.remove()) {
+            d->cerror(i18n("Could not remove index file %1", file.fileName()));
+        } else {
+            d->coutput(i18n("Removed %1", file.fileName()));
+        }
+    }
 }
 
 void PackageTool::packageInstalled(KJob *job)
@@ -649,13 +759,13 @@ void PackageTool::packageInstalled(KJob *job)
     bool success = (job->error() == KJob::NoError);
     int exitcode = 0;
     if (success) {
-        if (d->parser->isSet(QStringLiteral("upgrade"))) {
+        if (d->parser->isSet(Options::upgrade)) {
             d->coutput(i18n("Successfully upgraded %1", d->packageFile));
         } else {
             d->coutput(i18n("Successfully installed %1", d->packageFile));
         }
     } else {
-        d->coutput(i18n("Error: Installation of %1 failed: %2", d->packageFile, job->errorText()));
+        d->cerror(i18n("Error: Installation of %1 failed: %2", d->packageFile, job->errorText()));
         exitcode = 4;
     }
     exit(exitcode);
@@ -666,7 +776,7 @@ void PackageTool::packageUninstalled(KJob *job)
     bool success = (job->error() == KJob::NoError);
     int exitcode = 0;
     if (success) {
-        if (d->parser->isSet(QStringLiteral("upgrade"))) {
+        if (d->parser->isSet(Options::upgrade)) {
             d->coutput(i18n("Upgrading package from file: %1", d->packageFile));
             KJob *installJob = d->installer.install(d->packageFile, d->packageRoot);
             connect(installJob, SIGNAL(result(KJob*)), SLOT(packageInstalled(KJob*)));
@@ -674,7 +784,7 @@ void PackageTool::packageUninstalled(KJob *job)
         }
         d->coutput(i18n("Successfully uninstalled %1", d->packageFile));
     } else {
-        d->coutput(i18n("Error: Uninstallation of %1 failed: %2", d->packageFile, job->errorText()));
+        d->cerror(i18n("Error: Uninstallation of %1 failed: %2", d->packageFile, job->errorText()));
         exitcode = 7;
     }
     exit(exitcode);
